local api = require("gemini.api")
local M = {}

-- State for managing current suggestion and debounce timer
local current_suggestion = {
    text = nil,
    start_col = nil,
    namespace_id = vim.api.nvim_create_namespace('gemini_suggestion'),
    timer = nil,
    last_request = 0,
    cache = {},
    debounce_ms = 1000, -- Wait 1 second between requests
    min_chars = 0,      -- Minimum characters before triggering completion
    max_context_lines = 10, -- Maximum number of context lines to send
}

-- Helper function to get visible lines around cursor
local function get_visible_lines()
    local win = vim.api.nvim_get_current_win()
    local cursor_line = vim.api.nvim_win_get_cursor(0)[1]
    local top_line = vim.fn.line('w0')
    local bottom_line = vim.fn.line('w$')
    
    -- Get all visible lines
    local lines = vim.api.nvim_buf_get_lines(0, top_line - 1, bottom_line, false)
    local relative_cursor = cursor_line - top_line + 1
    
    return lines, relative_cursor
end

-- Helper function to get current line indent
local function get_line_indent(line)
    local indent = line:match("^%s+") or ""
    return indent
end

-- Debug function
local function debug_print(...)
    vim.notify(string.format(...), vim.log.levels.INFO)
end

-- Helper function to clear suggestion
local function clear_suggestion()
    if current_suggestion.timer then
        vim.fn.timer_stop(current_suggestion.timer)
        current_suggestion.timer = nil
    end
    
    if current_suggestion.text then
        vim.api.nvim_buf_clear_namespace(0, current_suggestion.namespace_id, 0, -1)
        current_suggestion.text = nil
        current_suggestion.start_col = nil
    end
end

-- Check if we should rate limit
local function should_rate_limit()
    local now = vim.loop.now()
    if (now - current_suggestion.last_request) < current_suggestion.debounce_ms then
        return true
    end
    current_suggestion.last_request = now
    return false
end

-- Cache key generation
local function get_cache_key(context)
    return vim.fn.sha256(context)
end

-- Helper function to show suggestion
local function show_suggestion(suggestion, start_col)
    clear_suggestion()
    
    local cursor = vim.api.nvim_win_get_cursor(0)
    local line = cursor[1] - 1
    local current_line = vim.api.nvim_get_current_line()
    
    -- Get current line indent
    local indent = get_line_indent(current_line)
    
    -- Split suggestion into lines and apply indent to all lines except first
    local suggestion_lines = vim.split(suggestion, "\n")
    for i = 2, #suggestion_lines do
        suggestion_lines[i] = indent .. suggestion_lines[i]
    end
    
    if #suggestion_lines == 0 then return end
    
    -- Show ghost text
    vim.api.nvim_buf_set_extmark(0, current_suggestion.namespace_id, line, start_col, {
        virt_text = {{suggestion_lines[1], 'GeminiSuggestion'}},
        virt_text_pos = 'overlay',
        hl_mode = 'combine',
    })
    
    -- Show remaining lines as virtual lines
    if #suggestion_lines > 1 then
        local virt_lines = {}
        for i = 2, #suggestion_lines do
            table.insert(virt_lines, {{suggestion_lines[i], 'GeminiSuggestion'}})
        end
        
        vim.api.nvim_buf_set_extmark(0, current_suggestion.namespace_id, line, start_col, {
            virt_lines = virt_lines,
            virt_lines_above = false,
        })
    end
    
    current_suggestion.text = table.concat(suggestion_lines, "\n")
    current_suggestion.start_col = start_col
end

function M.accept_suggestion()
    if not (current_suggestion.text and current_suggestion.start_col) then
        return false
    end

    local line = vim.api.nvim_win_get_cursor(0)[1] - 1
    local col = vim.api.nvim_win_get_cursor(0)[2]
    
    -- Only accept if we're still at or after the suggestion start
    if col < current_suggestion.start_col then
        return false
    end
    
    -- Get current line text
    local line_text = vim.api.nvim_buf_get_lines(0, line, line + 1, true)[1]
    local text_after_cursor = string.sub(line_text, col + 1)
    
    -- Split suggestion into lines
    local suggestion_lines = vim.split(current_suggestion.text, "\n")
    if #suggestion_lines == 0 then return false end
    
    -- For the first line, only insert the part that doesn't overlap
    local first_line = suggestion_lines[1]
    local common_length = 0
    while common_length < #text_after_cursor and common_length < #first_line do
        if string.sub(text_after_cursor, common_length + 1, common_length + 1) ==
           string.sub(first_line, common_length + 1, common_length + 1) then
            common_length = common_length + 1
        else
            break
        end
    end
    
    -- Only insert the non-overlapping part
    local to_insert = string.sub(first_line, common_length + 1)
    if to_insert ~= "" then
        vim.api.nvim_buf_set_text(
            0,
            line,
            col,
            line,
            col,
            {to_insert}
        )
    end
    
    -- Insert remaining lines below
    if #suggestion_lines > 1 then
        vim.api.nvim_buf_set_lines(
            0,
            line + 1,
            line + 1,
            false,
            vim.list_slice(suggestion_lines, 2)
        )
    end
    
    -- Move cursor to end of inserted text
    local final_line = line + #suggestion_lines - 1
    local final_col
    if final_line == line then
        final_col = col + #to_insert
    else
        final_col = #suggestion_lines[#suggestion_lines]
    end
    vim.api.nvim_win_set_cursor(0, {final_line + 1, final_col})
    
    -- Clear suggestion and cache for the current context
    clear_suggestion()
    current_suggestion.cache = {} -- Clear cache to force new suggestions
    
    -- Schedule next completion after a short delay
    vim.defer_fn(function()
        if vim.b.gemini_completion_enabled then
            M.trigger_completion()
        end
    end, 100)
    
    return true
end

-- Helper function to get relevant context around cursor
local function get_context_around_cursor(lines, current_line, max_lines)
    local context = {}
    local start_line = math.max(1, current_line - math.floor(max_lines/2))
    local end_line = math.min(#lines, current_line + math.floor(max_lines/2))
    
    for i = start_line, end_line do
        if lines[i] and #lines[i] > 0 then
            table.insert(context, string.format("L%d: %s", i, lines[i]))
        end
    end
    return context
end

function M.trigger_completion()
    if current_suggestion.timer then
        vim.fn.timer_stop(current_suggestion.timer)
    end
    
    current_suggestion.timer = vim.fn.timer_start(150, function()
        local cursor = vim.api.nvim_win_get_cursor(0)
        local line = cursor[1] - 1
        local col = cursor[2]
        
        -- Get visible context
        local visible_lines, relative_cursor = get_visible_lines()
        local current_line = visible_lines[relative_cursor]
        
        if not current_line or current_line == "" then
            clear_suggestion()
            return
        end
        
        -- Don't trigger in comments or strings
        local syntax_group = vim.fn.synIDattr(vim.fn.synID(cursor[1], col, 1), "name")
        if syntax_group:match("Comment") or syntax_group:match("String") then
            clear_suggestion()
            return
        end
        
        -- Prepare context for AI
        local context = table.concat(visible_lines, "\n")
        local file_type = vim.bo.filetype
        
        local prompt = string.format([[
In this %s file, complete the code at line %d, column %d:

%s

Return ONLY the completion text that would naturally continue from the cursor position.
Focus on completing the current statement or block.
Consider the visible context, syntax, and code style.
Do not repeat any text that appears before the cursor.]], 
            file_type, cursor[1], col + 1, context)
        
        -- Check cache and rate limiting
        local cache_key = get_cache_key(prompt)
        if current_suggestion.cache[cache_key] then
            show_suggestion(current_suggestion.cache[cache_key], col)
            return
        end
        
        if should_rate_limit() then return end
        
        -- Get completion from Gemini
        api.get_response(prompt, nil, function(response, error)
            if error then return end
            
            if type(response) == "string" and #response > 0 then
                response = vim.trim(response)
                current_suggestion.cache[cache_key] = response
                
                vim.schedule(function()
                    local new_cursor = vim.api.nvim_win_get_cursor(0)
                    if new_cursor[1] == cursor[1] and math.abs(new_cursor[2] - col) <= 1 then
                        show_suggestion(response, col)
                    end
                end)
            end
        end)
    end)
end

-- Setup function to create highlight group and keymaps
function M.setup()
    -- Create highlight group for suggestions
    vim.api.nvim_set_hl(0, 'GeminiSuggestion', {
        fg = '#666666',
        italic = true,
        blend = 15  -- Makes the ghost text slightly transparent
    })
    
    -- Map Tab to accept suggestion
    vim.keymap.set('i', '<Tab>', function()
        if not M.accept_suggestion() then
            vim.api.nvim_feedkeys(vim.api.nvim_replace_termcodes('<Tab>', true, true, true), 'n', true)
        end
    end, { expr = false, noremap = true })
    
    -- Set up autocommand for real-time completion
    vim.api.nvim_create_autocmd("TextChangedI", {
        pattern = "*",
        callback = function()
            if vim.b.gemini_completion_enabled then
                M.trigger_completion()
            end
        end
    })
    
    -- Clear suggestion on cursor move
    vim.api.nvim_create_autocmd({'CursorMovedI', 'CursorMoved'}, {
        pattern = "*",
        callback = function()
            clear_suggestion()
        end
    })
end

return M
